﻿#include "StdAfx.h"
#include "ChatSearchWnd.h"
#include "UIMenu.h"
#include "main_frame.hpp"
#include "ChatDlg.h"

DUI_BEGIN_MESSAGE_MAP(CChatSearchWnd, WindowImplBase, FALSE)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_WINDOWINIT, _OnNotifyPrepare)
    DUI_ON_MSGTYPE(DUI_MSGTYPE_ITEMCLICK, _OnNotifyItemClick)
    DUI_ON_CLICK_CTRNAME(kChatAddBtn, _OnCtrlChatAddBtnClick)
DUI_END_MESSAGE_MAP()

DUI_BEGIN_CUSTOM_MSG_MAP(CChatSearchWnd)
    ON_CUSTOM_MSG(WM_KEYDOWN, _OnMsgKeyDown)
    ON_CUSTOM_MSG(WM_SYSKEYDOWN, _OnMsgSysKeyDown)
    ON_CUSTOM_MSG(WM_SIZE, _OnMsgSize)
    ON_CUSTOM_MSG(MSG_ACTIVE_PRARENT, _OnMsgActiveParent)
    ON_CUSTOM_MSG(MSG_INACTIVE_HIDE, _OnMsgInactiveHide)
DUI_END_CUSTOM_MSG_MAP()


CChatSearchWnd::CChatSearchWnd(CChatDlg *pChatDlg)
{
	m_pChatDlg = pChatDlg;
	m_nChatSearchHeight = 148;
    m_rcClient.SetRectEmpty();

}

CChatSearchWnd::~CChatSearchWnd()
{
    m_pChatDlg = NULL;
}

LPCTSTR CChatSearchWnd::GetWindowClassName() const
{
	return _T("ChatSearchWnd");
}

void CChatSearchWnd::OnFinalMessage(HWND hWnd)
{
	CChatSearchList* pFriendsList = GetFriendsCtrl();
	if (pFriendsList != NULL)
	{
		if (pFriendsList->GetCount() > 0)
		{
			pFriendsList->RemoveAll();
		}
	}
	WindowImplBase::OnFinalMessage(hWnd);
	if (m_pChatDlg != NULL)
	{
		::PostMessage(m_pChatDlg->GetHWND(), MSG_CLOSE_CHAT_SEARCH, 0, 0);
	}
	delete this;
	
}

void CChatSearchWnd::_OnCtrlChatAddBtnClick(TNotifyUI& msg)
{
    Node* node = (Node*)msg.pSender->GetTag();
    ::PostMessage(m_pChatDlg->GetHWND(), WM_ADD_USER_CHAT, (WPARAM)node, 0);
}

void CChatSearchWnd::_OnNotifyItemClick(TNotifyUI& msg)
{
    CChatSearchList* pFriendsList = static_cast<CChatSearchList*>(m_PaintManager.FindControl(kSearchList));
    if ((pFriendsList != NULL) &&  pFriendsList->GetItemIndex(msg.pSender) != -1)
    {
        if (_tcsicmp(msg.pSender->GetClass(), _T("ListContainerElementUI")) == 0)
        {
            Node* node = (Node*)msg.pSender->GetTag();
            if (msg.wParam == UIEVENT_RBUTTONUP)
            {
                if (!node->folder())
                {
                    CPoint point = msg.ptMouse;
                    ClientToScreen(m_hWnd, &point);
                }
            }
        }
    }
}

LRESULT CChatSearchWnd::_OnMsgKeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    bHandled = FALSE;
    if ((TCHAR)wParam == VK_ESCAPE)
    {
        if (m_pChatDlg != NULL)
        {
            m_pChatDlg->CloseChatSearchWnd(TRUE, TRUE);
            bHandled = TRUE;
        }
    }
	return 0;
}

LRESULT CChatSearchWnd::_OnMsgSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    if (m_pChatDlg != NULL)
    {
        m_pChatDlg->ReSizeCtrl();
    }
	bHandled = FALSE;
    return 0;
}

LRESULT CChatSearchWnd::_OnMsgActiveParent(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	SetFocus(m_hWnd);
    return 0;
}

LRESULT CChatSearchWnd::_OnMsgInactiveHide(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    CPoint pt(0, 0);
    GetCursorPos(&pt);
    CRect rcClient(0, 0, 0, 0);
    GetWindowRect(m_hWnd, rcClient);
    CRect rcSearchBtn(3, rcClient.bottom, 25, rcClient.bottom + 30);
    CRect rcProfile(0, 0, 0, 0);
    BOOL bOnProfile = FALSE;
    if (g_pProfileWnd != NULL && IsWindow(g_pProfileWnd->GetHWND()) && g_pProfileWnd->GetParentHwnd() == m_hWnd)
    {
        GetWindowRect(g_pProfileWnd->GetHWND(), rcProfile);
        if (rcProfile.PtInRect(pt))
        {
            bOnProfile = TRUE;
        }
    }
    if (m_pChatDlg != NULL && !rcClient.PtInRect(pt) && !rcSearchBtn.PtInRect(pt) && !bOnProfile)
    {
        m_pChatDlg->CloseChatSearchWnd(TRUE);
    }
    return 0;
}

LRESULT CChatSearchWnd::_OnMsgSysKeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    bHandled = FALSE;
    if (GetKeyState(VK_MENU) < 0 && VK_F4 == wParam)
    {
        if (m_pChatDlg != NULL)
        {
            m_pChatDlg->CloseChatSearchWnd(TRUE, TRUE);
            bHandled = TRUE;
        }
    }
    return 0;
}

CControlUI* CChatSearchWnd::CreateControl(LPCTSTR pstrClass)
{
	if (_tcsicmp(pstrClass, _T("ChatSearchUI")) == 0)
	{
		return new CChatSearchList(m_PaintManager, m_hWnd);
	}

	return NULL;
}

void CChatSearchWnd::SetClientRect(CRect rc)
{
	m_rcClient = rc;
	MoveWindow(GetHWND(), m_rcClient.left, m_rcClient.top, m_rcClient.Width(), m_rcClient.Height(), TRUE);
}

int CChatSearchWnd::GetChatSearchHeight()
{
	return m_nChatSearchHeight;
}

void CChatSearchWnd::SetChatSearchHeight(int nHeight, BOOL bResize)
{
	m_nChatSearchHeight = nHeight;
	if (bResize)
	{
		CRect rcSearch(0, 0, 0, 0);
		GetClientRect(GetHWND(), &rcSearch);
		rcSearch.top = rcSearch.bottom - m_nChatSearchHeight;
		SetClientRect(rcSearch);
		m_pChatDlg->ReSizeCtrl(TRUE);
	}
}

void CChatSearchWnd::InitWindow()
{
	MoveWindow(GetHWND(), m_rcClient.left, m_rcClient.top, m_rcClient.Width(), m_rcClient.Height(), TRUE);
}

void CChatSearchWnd::_OnNotifyPrepare(TNotifyUI& msg)
{
	//UpdateFriendsList();
}

CDuiString CChatSearchWnd::GetSkinFile()
{
	return _T("ChatSearchWnd.xml");
}

CDuiString CChatSearchWnd::GetSkinFolder()
{
	return _T("skin\\nTalkRes");
}

LRESULT CChatSearchWnd::OnKillFocus(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	CPoint pt(0, 0);
	GetCursorPos(&pt);
	CRect rcClient(0, 0, 0, 0);
	GetWindowRect(m_hWnd, rcClient);
	CRect rcSearchBtn(rcClient.left + 3, rcClient.bottom, rcClient.left + 25, rcClient.bottom + 30);
	CRect rcProfile(0, 0, 0, 0);
	BOOL bOnProfile = FALSE;
	if (g_pProfileWnd != NULL && IsWindow(g_pProfileWnd->GetHWND()) && g_pProfileWnd->GetParentHwnd() == m_hWnd)
	{
		GetWindowRect(g_pProfileWnd->GetHWND(), rcProfile);
		if (rcProfile.PtInRect(pt))
		{
			bOnProfile = TRUE;
		}
	}
	if (m_pChatDlg != NULL && !rcClient.PtInRect(pt) && !rcSearchBtn.PtInRect(pt) && !bOnProfile)
	{
		m_pChatDlg->CloseChatSearchWnd(TRUE);
	}

	return 0;
}

CChatSearchList* CChatSearchWnd::GetFriendsCtrl()
{
	CChatSearchList* pFriendsList = static_cast<CChatSearchList*>(m_PaintManager.FindControl(kSearchList));
	return pFriendsList;
}

std::vector<CString> CChatSearchWnd::GetLeftListUserID()
{
	std::vector<CString> vecUserID;
	CChatLeftList* pLeftList = m_pChatDlg->GetLeftList();
	if (pLeftList != NULL)
	{
		Node* node = NULL;
		for (int i = 0; i < pLeftList->GetRoot()->num_children(); i++)
		{
			node = pLeftList->GetRoot()->child(i);
			if(node != NULL)
			{
				vecUserID.push_back(node->data().value.GetData());
			}
		}
	}
	return vecUserID;
}

CLabelUI* CChatSearchWnd::GetNosearchResultLabel()
{
	return static_cast<CLabelUI*>(m_PaintManager.FindControl(kNosearchResult));
}

LRESULT CChatSearchWnd::OnNcHitTest(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	int ret = __super::OnNcHitTest(uMsg, wParam, lParam, bHandled);
	if(HTTOP != ret)
	{
		return HTCLIENT;
	}
	return ret;
}